#ifndef _CHRONOPTICS_TOF_CALIBRATION_H_
#define _CHRONOPTICS_TOF_CALIBRATION_H_

#include <chronoptics/tof/data.h>

#ifdef __cplusplus
extern "C" {
#endif

/** 2D point for indexing into array
 */
typedef struct tof_point_2d* tof_point_2d_t;

/** Destruct tof_point_2d */
TOF_EXPORT void tof_point_2d_delete(tof_point_2d_t ptr);

/** Get the row value
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Row value
 */
TOF_EXPORT int32_t tof_point_2d_row(tof_point_2d_t ptr, tof_error_t *error);

/** Get the column value
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Column value
 */
TOF_EXPORT int32_t tof_point_2d_col(tof_point_2d_t ptr, tof_error_t *error);

/** This class contains all the calibration information
 */
typedef struct tof_calibration* tof_calibration_t;

/** Destruct tof_calibration */
TOF_EXPORT void tof_calibration_delete(tof_calibration_t ptr);

/** Read calibration from disk
 * @param ptr Pointer to class
 * @param file_location Location of calibration file
 * @param error Pointer to error
 * @return New object
 */
TOF_EXPORT tof_calibration_t tof_calibration_new_from_disk(const char* file_location, tof_error_t *error);

/** Write calibration to disk
 * @param ptr Pointer to class
 * @param file_location Location to save the calibration file to
 * @param error Pointer to error
 */
TOF_EXPORT void tof_calibration_write(const tof_calibration_t ptr, const char* file_location, tof_error_t *error);

/** The the calibrated frequencies in the calibration file
 * @param ptr Pointer to class
 * @param calibrated_frequencies The calibrated modulation frequencies
 * @param capacity The amount of data the above pointer can hold
 * @param error Pointer to error
 * @return Number of pointers filled or complete array size
 */
TOF_EXPORT size_t tof_calibration_get_calibrated_frequencies(const tof_calibration_t ptr, float* calibrated_frequencies, size_t capacity, tof_error_t *error);

/** Get focal length x
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Focal length
 */
TOF_EXPORT double tof_calibration_get_focal_length_x(const tof_calibration_t ptr, tof_error_t *error);

/** Get focal length y
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Focal length
 */
TOF_EXPORT double tof_calibration_get_focal_length_y(const tof_calibration_t ptr, tof_error_t *error);

/** Get principal point x
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Principal point
 */
TOF_EXPORT double tof_calibration_get_principal_point_x(const tof_calibration_t ptr, tof_error_t *error);

/** Get principal point y
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Principal point
 */
TOF_EXPORT double tof_calibration_get_principal_point_y(const tof_calibration_t ptr, tof_error_t *error);

/** Get the camera matrix of the depth sensor and lens
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return[9] Camera matrix, 3x3 matrix
 */
TOF_EXPORT double* tof_calibration_get_depth_camera_matrix(tof_calibration_t ptr, tof_error_t *error);

/** Get the distortion coefficients of the depth lens
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return[5] Distortion coefficients, 5x1 matrix
 */
TOF_EXPORT double* tof_calibration_get_depth_distortion_coefficients(tof_calibration_t ptr, tof_error_t *error);

/** Get the camera matrix of the rgb sensor and lens
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return[9] Camera matrix, 3x3 matrix
 */
TOF_EXPORT double* tof_calibration_get_rgb_camera_matrix(tof_calibration_t ptr, tof_error_t *error);

/** Get the distortion coefficients of the rgb lens
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return[5] Distortion coefficients, 5x1 matrix
 */
TOF_EXPORT double* tof_calibration_get_rgb_distortion_coefficients(tof_calibration_t ptr, tof_error_t *error);

/** Get the rotation vector between the depth and rgb sensor
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return[3] Rotation vector, 3x1 matrix
 */
TOF_EXPORT double* tof_calibration_get_rotation_vector(tof_calibration_t ptr, tof_error_t *error);

/** Get the translation vector between the depth and rgb sensor
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return[3] Translation vector, 3x1 matrix
 */
TOF_EXPORT double* tof_calibration_get_translation_vector(tof_calibration_t ptr, tof_error_t *error);

/** Transform a 3D point into a 2D pixel coordinate of the depth camera
 * @param x X value
 * @param y Y value
 * @param z Z value
 * @param calibration The camera calibration
 * @param error Pointer to error
 * @return 2D pixel coordinate of the depth camera
 */
TOF_EXPORT tof_point_2d_t tof_transform_3d_to_depth_2d(float x, float y, float z, tof_calibration_t calibration, tof_error_t *error);

/** Transform a 3D point into a 2D pixel coordinate of the color camera
 * @param x X value
 * @param y Y value
 * @param z Z value
 * @param calibration The camera calibration
 * @param error Pointer to error
 * @return 2D pixel coordinate of the color camera
 */
TOF_EXPORT tof_point_2d_t tof_transform_3d_to_color_2d(float x, float y, float z, tof_calibration_t calibration, tof_error_t *error);

/** Transform a 2D pixel coordinate of the color camera to the 2D pixel
 * coordinate of the depth camera.
 * @param row Row coordinate
 * @param col Column cooridnate
 * @param depth_data Depth data containing xyz information, XYZ, XYZAmp, XYZBGR
 * and XYZBGRI are all valid
 * @param calibration The camera calibration
 * @param error Pointer to error
 * @return 2D pixel coordinate of the depth camera
 */
TOF_EXPORT tof_point_2d_t tof_transform_color_2d_to_depth_2d(int32_t row, int32_t col, tof_data_t depth_data, tof_calibration_t calibration, tof_error_t *error);

#ifdef __cplusplus
}
#endif

#endif
