#ifndef _CHRONOPTICS_TOF_DATA_H_
#define _CHRONOPTICS_TOF_DATA_H_

#include <chronoptics/tof/stream.h>

#ifdef __cplusplus
extern "C" {
#endif

/** The different matrix types
 */
enum tof_mat_type {
  TOF_MAT_TYPE_UINT8 = 0, /** A single unsigned byte */
  TOF_MAT_TYPE_INT8 = 1, /** A single signed byte */
  TOF_MAT_TYPE_UINT16 = 2, /** Two unsigned bytes */
  TOF_MAT_TYPE_INT16 = 3, /** Two signed bytes */
  TOF_MAT_TYPE_INT32 = 4, /** Four signed bytes */
  TOF_MAT_TYPE_FLOAT = 5, /** Four bytes floating point value */
  TOF_MAT_TYPE_DOUBLE = 6, /** Eight bytes floating point value */
  TOF_MAT_TYPE_FLOAT_4 = 29, /** Four bytes floating point value with four channels */
  TOF_MAT_TYPE_FLOAT_8 = 61, /** Four bytes floating point value with eight channels */
};

/** Function signature of callback that's called after a tof_data object is
 * destructed containing a user pointer
 * @param pointer The pointer to the data to write into
 * @param capacity The amount of data the pointer can hold
 * @param user_data The passed back user data
 */
typedef void (*tof_user_pointer_destructed_t)(uint8_t* pointer, size_t capacity, void* user_data);

/** This is the class that contains depth or image data
 */
typedef struct tof_data* tof_data_t;

/** Destruct tof_data */
TOF_EXPORT void tof_data_delete(tof_data_t ptr);

/** Get pointer to data
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return 
 */
TOF_EXPORT void* tof_data_data(const tof_data_t ptr, tof_error_t *error);

/** Get size of the data
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The size
 */
TOF_EXPORT size_t tof_data_size(const tof_data_t ptr, tof_error_t *error);

/** Get user data that will get saved to csf file
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Set user data
 */
TOF_EXPORT uint64_t tof_data_get_user_data(const tof_data_t ptr, tof_error_t *error);

/** Set user data that will get saved to csf file
 * @param ptr Pointer to class
 * @param user_data Set user data
 * @param error Pointer to error
 */
TOF_EXPORT void tof_data_set_user_data(tof_data_t ptr, uint64_t user_data, tof_error_t *error);

/** Get the matrix type
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The matrix type
 */
TOF_EXPORT enum tof_mat_type tof_data_mat_type(const tof_data_t ptr, tof_error_t *error);

/** Get the number of rows
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Number of rows
 */
TOF_EXPORT int32_t tof_data_rows(const tof_data_t ptr, tof_error_t *error);

/** Get the number of columns
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Number of columns
 */
TOF_EXPORT int32_t tof_data_cols(const tof_data_t ptr, tof_error_t *error);

/** The frame counter
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Frame count
 */
TOF_EXPORT uint16_t tof_data_frame_count(const tof_data_t ptr, tof_error_t *error);

/** Get the frame type
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The frame type
 */
TOF_EXPORT enum tof_frame_type tof_data_frame_type(const tof_data_t ptr, tof_error_t *error);

/** Get the frame id
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The frame id
 */
TOF_EXPORT uint32_t tof_data_frame_id(const tof_data_t ptr, tof_error_t *error);

/** Get the modulation frequency of the frame
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The modulation frequency
 */
TOF_EXPORT float tof_data_modulation_frequency(const tof_data_t ptr, tof_error_t *error);

/** Get the integration time of the frame
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The integration time
 */
TOF_EXPORT uint32_t tof_data_integration_time(const tof_data_t ptr, tof_error_t *error);

/** Get a bit array of the processing performed on the frame
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return A bit array
 */
TOF_EXPORT uint32_t tof_data_process(const tof_data_t ptr, tof_error_t *error);

/** Get the scale used for integer frames, to get the original value divide by
 * scale
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The scale
 */
TOF_EXPORT float tof_data_scale(const tof_data_t ptr, tof_error_t *error);

/** Get the addition used for integer frames, to get the original value
 * substract the addition
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The addition
 */
TOF_EXPORT float tof_data_addition(const tof_data_t ptr, tof_error_t *error);

/** Get the temperature of various parts of the camera
 * @param ptr Pointer to class
 * @param temperatures Temperatures
 * @param capacity The amount of data the above pointer can hold
 * @param error Pointer to error
 * @return Number of pointers filled or complete array size
 */
TOF_EXPORT size_t tof_data_temperatures(const tof_data_t ptr, float* temperatures, size_t capacity, tof_error_t *error);

/** Get the synchronization count
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Sync count
 */
TOF_EXPORT uint8_t tof_data_sync_count(const tof_data_t ptr, tof_error_t *error);

/** Check whether the data uses a pointer supplied by the user
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Data allocated by user
 */
TOF_EXPORT bool tof_data_user_allocated(const tof_data_t ptr, tof_error_t *error);

/** The user data that was given in the add_user_pointer function
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return User pointer
 */
TOF_EXPORT void* tof_data_user_pointer(const tof_data_t ptr, tof_error_t *error);

/** The acceleration measured by the imu
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return[3] Acceleration, x y z
 */
TOF_EXPORT int16_t* tof_data_acceleration(const tof_data_t ptr, tof_error_t *error);

/** The angular velocity measured by the imu
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return[3] Angular velocity, x y z
 */
TOF_EXPORT int16_t* tof_data_angular_velocity(const tof_data_t ptr, tof_error_t *error);

/** Unix time stamp since epoch
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Time stamp
 */
TOF_EXPORT uint64_t tof_data_time_stamp(tof_data_t ptr, tof_error_t *error);

#ifdef __cplusplus
}
#endif

#endif
