#include <chronoptics/tof.hpp>

#include "cxxopts.hpp"

namespace tof = chronoptics::tof;

struct Options {
  std::string serial;
  std::string file_name;
  bool download;
  bool calibration;
};

int main(int argc, char **argv) {
  cxxopts::Options arg_options("ToF file_type Exchange",
                               "Upload/Download files from chronoptics camera");

  Options options{};
  uint16_t port = 0;
  // clang-format off
  arg_options.add_options()
    ("h, help", "Help")
    ("list", "List cameras on network")
    ("s, serial", "Serial number of camera to connect to", cxxopts::value<std::string>(options.serial))
    ("p, port", "Port to talk on", cxxopts::value<uint16_t>(port))
    ("f, file", "The file name", cxxopts::value<std::string>(options.file_name))
    ("d, download", "Download")
    ("u, upload", "Upload")
    ("config", "Exchange config")
    ("calibration", "Exchange calibration")
  ;
  // clang-format on

  auto result = arg_options.parse(argc, argv);

  if (result.count("h") || result.count("help")) {
    std::cout << arg_options.help() << std::endl;
    return 0;
  }

  if (result.count("list")) {
    auto cameras = tof::discover_kea_cameras();

    for (auto &cam : cameras) {
      std::cout << "Camera: " << cam.serial() << " info: " << cam.info()
                << "\n";
    }

    return 0;
  }

  if (result.count("d") + result.count("u") != 1) {
    std::cout << "Specify one -u or -d\n";
    return -1;
  }

  if (!result.count("serial")) {
    std::cout << "Please specify camera serial\n";
    return -1;
  }

  options.download = result.count("d") > 0;

  if (result.count("config") + result.count("calibration") != 1) {
    std::cout
        << "Specify one of the following files: --config, --calibration\n";
    return -1;
  }

  options.calibration = result.count("calibration") > 0;

  try {
    tof::ProcessingConfig config;
    config.set_calibration_enabled(false);

    tof::KeaCamera cam(config, options.serial);

    std::cout << "Connected to camera " << options.serial << "\n";

    if (options.calibration) {
      if (options.download) {
        std::cout << "Downloading calibration file to " << options.file_name
                  << "\n";
        tof::Calibration cal = cam.get_calibration();
        cal.write(options.file_name);
      } else {
#ifdef TOF_INTERNAL
        std::cout << "Uploading " << options.file_name << "\n";
        tof::Calibration cal(options.file_name);
        cam.set_calibration(cal);
#else
        std::cout << "Uploading calibration file to camera is not supported\n";
#endif
      }
    } else {
      if (options.download) {
        std::cout << "Downloading configuration file to " << options.file_name
                  << "\n";
        tof::CameraConfig cam_config = cam.get_camera_config();
        cam_config.write(options.file_name);
      } else {
        std::cout << "Uploading " << options.file_name << "\n";
        tof::CameraConfig cam_config(options.file_name);
        cam.set_camera_config(cam_config);
      }
    }

    std::cout << "Success\n";
  } catch (std::exception &e) {
    std::cout << "Failed with " << e.what() << "\n";
  }

  return 0;
}
